import java.io.*;

public class Sudoku {
    public static final int QSIZE    = 3;
    public static final int SIZE     = QSIZE * QSIZE;
    public static final int CELLS    = SIZE * SIZE;
    public static final int FULL_SET = (1 << (SIZE + 1)) - 2;
    public static final boolean USE_BLOCKS   = true;
    public static final boolean USE_DIAGONAL = false;

    static final byte[] resolve;

    int cells;
    byte[][] field;
    int[] hline;
    int[] vline;
    int[] block;
    int dline1;
    int dline2;

    static {
        resolve = new byte[1 << (SIZE + 1)];
        for (int i = 1; i <= SIZE; i++) {
            resolve[FULL_SET - (1 << i)] = (byte)i;
        }
    }

    public Sudoku() {
        cells = 0;
        field = new byte[SIZE][SIZE];
        hline = new int[SIZE];
        vline = new int[SIZE];
        block = new int[SIZE];
        dline1 = 0;
        dline2 = 0;
    }

    Sudoku(Sudoku prototype) {
        cells = prototype.cells;
        field = new byte[SIZE][];
        for (int i = 0; i < SIZE; i++) {
            field[i] = prototype.field[i].clone();
        }
        hline = prototype.hline.clone();
        vline = prototype.vline.clone();
        block = prototype.block.clone();
        dline1 = prototype.dline1;
        dline2 = prototype.dline2;
    }

    int setFor(int x, int y) {
        if (field[x][y] != 0) {
            return 0;
        }
        int set = hline[x] | vline[y] | block[x / QSIZE + y - y % QSIZE];
        if (x == y) {
            set |= dline1;
        }
        if (x + y == SIZE - 1) {
            set |= dline2;
        }
        return set;
    }

    void update(int x, int y, byte v) {
        assert v <= SIZE;
        cells++;
        field[x][y] = v;

        int vbit = 1 << v;
        assert (hline[x] & vbit) == 0;
        hline[x] |= vbit;
        assert (vline[y] & vbit) == 0;
        vline[y] |= vbit;
        if (USE_BLOCKS) {
            assert (block[x / QSIZE + y - y % QSIZE] & vbit) == 0;
            block[x / QSIZE + y - y % QSIZE] |= vbit;
        }
        if (USE_DIAGONAL && x == y) {
            assert (dline1 & vbit) == 0;
            dline1 |= vbit;
        }
        if (USE_DIAGONAL && x + y == SIZE - 1) {
            assert (dline2 & vbit) == 0;
            dline2 |= vbit;
        }
    }

    public void read(InputStream in) throws IOException {
        BufferedReader br = new BufferedReader(new InputStreamReader(in));
        for (int y = 0; y < SIZE; y++) {
            String s = br.readLine();
            int len = s.length();
            int x = 0;
            for (int i = 0; i < len; i++) {
                char c = s.charAt(i);
                if (c != ' ') {
                    if (c >= '0' && c <= '9') {
                        update(x, y, (byte)(c - '0'));
                    }
                    x++;
                }
            }
        }
    }

    public void print(PrintStream out) {
        for (int y = 0; y < SIZE; y++) {
            for (int x = 0; x < SIZE; x++) {
                if ((x % QSIZE) == 0) {
                    out.print(' ');
                }
                byte v = field[x][y];
                out.print(v == 0 ? "." : Byte.toString(v));
            }
            out.println();
        }
        out.println();
    }

    public Sudoku solve() {
        // Trivial excluding
        int oldCells;
        do {
            oldCells = cells;
            for (int x = 0; x < SIZE; x++) {
                for (int y = 0; y < SIZE; y++) {
                    byte v = resolve[setFor(x, y)];
                    if (v > 0) {
                        update(x, y, v);
                    }
                }
            }
        } while (cells != oldCells);

        // Found a solution?
        if (cells == CELLS) {
            return this;
        }

        // Attempt to guess one cell
        for (int x = 0; x < SIZE; x++) {
            for (int y = 0; y < SIZE; y++) {
                int set = setFor(x, y);
                if (set != 0) {
                    for (int v = 1; v <= SIZE; v++) {
                        if ((set & (1 << v)) == 0) {
                            Sudoku candidate = new Sudoku(this);
                            candidate.update(x, y, (byte)v);
                            if ((candidate = candidate.solve()) != null) {
                                return candidate;
                            }
                        }
                    }
                    return null;
                }
            }
        }
        return null;
    }

    public static void main(String[] args) {
        if (args.length == 0) {
            System.out.println("Usage: java Sudoku <filename>");
            return;
        }

        Sudoku s = new Sudoku();
        try {
            FileInputStream fis = new FileInputStream(args[0]);
            s.read(fis);
            fis.close();
        } catch (Exception e) {
            System.out.println("Unable to read input file");
            e.printStackTrace();
            return;
        }

        s.print(System.out);
        s = s.solve();
        if (s == null) {
            System.out.println("Cannot be resolved");
        } else {
            s.print(System.out);
        }
    }

}
